/*******************************************************************************
 * Copyright (c) 2008 Hallvard Traetteberg.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Hallvard Traetteberg - initial API and implementation
 ******************************************************************************/
package org.eclipse.e4.tm.builder;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eclipse.e4.tm.builder.swt.SwtBuilder;
import org.eclipse.e4.tm.stringconverter.StringConversion;
import org.eclipse.e4.tm.stringconverters.SimpleObjectStringConverter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.widgets.Display;

public class StringConverterTest extends TestCase {

	private StringConversion stringConverter;
	private AbstractBuilder builder;

	private Display display;

	protected void setUp() throws Exception {
		super.setUp();
		builder = new SwtBuilder();
		stringConverter = builder.getStringConverter();
		stringConverter.registerStringConverter(Class.class, builder.getClassResolver());
		builder.getClassResolver().importPackage("org.eclipse.swt.widgets");
		builder.getClassResolver().importPackage("org.eclipse.swt.layout");
		display = Display.getDefault();
	}

	protected <T> void testStringConverter(Class<T> klass, String source, Object expected) {
		testStringConverter(klass, klass, source, expected);
	}
	protected <T> void testStringConverter(Class<T> type, Class<?> klass, String source, Object expected) {
		T result = null;
		try {
			result = stringConverter.convert(source, type);
		} catch (Exception e) {
			Assert.assertEquals(expected, e.getClass());
		}
		if (! klass.isInstance(result)) {
			Assert.fail(result + " is not of " + klass);
		}
		if (klass.isArray()) {
			int length = Array.getLength(result);
			for (int i = 0; i < length; i++) {
				Assert.assertEquals(Array.get(expected, i), Array.get(result, i));
			}
		} else if (! klass.isInstance(expected)) {
			Assert.assertEquals(expected.toString(), result.toString());
		} else {
			Assert.assertEquals(expected, result);
		}
	}

	public void testBooleanStringConverter() {
		testStringConverter(Boolean.TYPE, Boolean.class, "true", Boolean.TRUE);
		testStringConverter(Boolean.TYPE, Boolean.class, "false", Boolean.FALSE);
	}
	public void testIntStringConverter() {
		testStringConverter(Integer.TYPE, Integer.class, "0", new Integer(0));
		testStringConverter(Integer.TYPE, Integer.class, "10", new Integer(10));
	}
	public void testByteStringConverter() {
		testStringConverter(Byte.TYPE, Byte.class, "0", new Byte((byte)0));
		testStringConverter(Byte.TYPE, Byte.class, "10", new Byte((byte)10));
	}
	public void testDoubleStringConverter() {
		testStringConverter(Double.TYPE, Double.class, "1.2", new Double(1.2));
	}
	public void testLongStringConverter() {
		testStringConverter(Long.TYPE, Long.class, "0", new Long(0));
	}
	public void testFloatStringConverter() {
		testStringConverter(Float.TYPE, Float.class, "1.2", new Float(1.2));
	}
	public void testStringStringConverter() {
		testStringConverter(String.class, "hello", "hello");
	}
	public void testStringArrayStringConverter() {
		testStringConverter(String[].class, "hello world", new String[]{"hello", "world"});
		// tests different parentheses
		testStringConverter(String[].class, "(hello, world)", new String[]{"hello", " world"});
		testStringConverter(String[].class, "[hello world]", new String[]{"hello", "world"});
		testStringConverter(String[].class, "{hello ;world}", new String[]{"hello ", "world"});
		testStringConverter(String[].class, "<hello | world>", new String[]{"hello ", " world"});
	}
	public void testStringBufferStringConverter() {
		testStringConverter(StringBuffer.class, "hello", "hello");
	}
	public void testIntArrayStringConverter() {
		testStringConverter(int[].class, "1 2 3", new int[]{1, 2, 3});
	}
	public void testByteArrayStringConverter() {
		testStringConverter(byte[].class, "1 2 3", new byte[]{1, 2, 3});
	}
	public void testRGBStringConverter() {
		testStringConverter(RGB.class, "0 1 2", new RGB(0, 1, 2));
		testStringConverter(RGB.class, "-1 128 256", new RGB(0, 128, 255));
		testStringConverter(RGB.class, "-1 256", new RGB(0, 255, 0));
		testStringConverter(RGB.class, "#00FFFF", new RGB(0, 255, 255));
	}

	public void testPointStringConverter() {
		testStringConverter(Point.class, "0 1", new Point(0, 1));
	}

	public void testRectangleStringConverter() {
		testStringConverter(Rectangle.class, "0 1 2 3", new Rectangle(0, 1, 2, 3));
	}

	private void testColorStringConverter(String colorString, Color c) {
		Color result = null;
		try {
			result = stringConverter.convert(colorString, Color.class);
			assertEquals(c, result);
		} finally {
			if (result != null) {
				result.dispose();
			}
			c.dispose();
		}
	}
	public void testColorStringConverter() {
		testColorStringConverter("white", new Color(display, 255, 255, 255));
		testColorStringConverter("red", new Color(display, 255, 0, 0));
	}

	private void testFontStringConverter(String fontString, Font f) {
		Font result = null;
		try {
			result = stringConverter.convert(fontString, Font.class);
			FontData[] fontData1 = result.getFontData();
			FontData[] fontData2 = f.getFontData();
			assertTrue(Arrays.equals(fontData1, fontData2));
		} finally {
			if (result != null) {
				result.dispose();
			}
			f.dispose();
		}
	}
	public void testFontStringConverter() {
		testFontStringConverter("Times 12", new Font(display, "Times", 12, SWT.NONE));
		testFontStringConverter("bold italic Times 18", new Font(display, "Times", 18, SWT.BOLD | SWT.ITALIC));
	}

	public void testURLStringConverter() {
		try {
			testStringConverter(URL.class, "http://www.news.com/", new URL("http://www.news.com/"));
		} catch (MalformedURLException e) {
		}
	}

	public void testClassStringConverter() {
		testStringConverter(Class.class, "int", int.class);
		testStringConverter(Class.class, "int[]", int[].class);
		testStringConverter(Class.class, "Class", Class.class);
		testStringConverter(Class.class, "Class[]", Class[].class);
	}

	protected void testMethodStringConverter(String source, Class<?> klass, String methodName, Class<?>[] args) {
		Method m = null;
		try {
			m = klass.getMethod(methodName, args);
		} catch (SecurityException e) {
		} catch (NoSuchMethodException e) {
		}
		Assert.assertNotNull(m);
		testStringConverter(Method.class, source, m);
	}

	public void testSimpleObjectStringConverter() {
		stringConverter.registerStringConverter(RowData.class, new SimpleObjectStringConverter());
		String source = "RowData {int: width = 50; int: height = 25; boolean: exclude = true;}";
		Object result = null;
		try {
			result = stringConverter.convert(source, RowData.class);
		} catch (Exception e) {
			Assert.fail("Couldn't create RowData from " + source);
		}
		Assert.assertNotNull(result);
		RowData rowData = (RowData)result;
		Assert.assertEquals(50, rowData.width);
		Assert.assertEquals(25, rowData.height);
		Assert.assertTrue(rowData.exclude);
	}

	public static void main(String[] args) {
		TestRunner runner = new TestRunner();
		runner.doRun(new TestSuite(StringConverterTest.class));
	}
}
